@@ -13,8 +13,8 @@
 package No::Worries::File;
 use strict;
 use warnings;
-our $VERSION  = "1.3";
-our $REVISION = sprintf("%d.%02d", q$Revision: 1.16 $ =~ /(\d+)\.(\d+)/);
+our $VERSION  = "1.4";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.18 $ =~ /(\d+)\.(\d+)/);
 
 #
 # used modules
@@ -32,32 +32,38 @@ use Params::Validate qw(validate :types);
 our($DefaultBufSize);
 
 #
-# binmode() helper
+# open() helper
 #
 
-sub _binmode ($$$) {
-    my($path, $fh, $mode) = @_;
+sub _open ($$$) {
+    my($path, $mode, $opt) = @_;
+    my($fh);
 
-    if ($mode eq "utf8") {
-        binmode($fh, ":encoding(utf8)")
-            or dief("cannot binmode(%s, :encoding(utf8)): %s", $path, $!);
-    } else {
+    return($opt->{handle}) if $opt->{handle};
+    ## no critic 'InputOutput::RequireBriefOpen'
+    open($fh, $mode, $path) or dief("cannot open(%s): %s", $path, $!);
+    if ($opt->{binmode}) {
+        binmode($fh, $opt->{binmode})
+            or dief("cannot binmode(%s, %s): %s", $path, $opt->{binmode}, $!);
+    } elsif ($opt->{binary}) {
         binmode($fh)
             or dief("cannot binmode(%s): %s", $path, $!);
     }
+    return($fh);
 }
 
 #
-# read from a file
+# sysread() helper
 #
 
-sub _file_read_io ($$$$) {
+sub _read ($$$$) {
     my($path, $fh, $data, $bufsize) = @_;
     my($done, $ref, $result);
 
     $done = -1;
     $ref = $data ? ref($data) : "";
     if ($ref eq "SCALAR") {
+        # by reference
         ${ $data } = "";
         while ($done) {
             $done = sysread($fh, ${ $data }, $bufsize, length(${ $data }));
@@ -66,6 +72,7 @@ sub _file_read_io ($$$$) {
         }
         $result = $data;
     } elsif ($ref eq "CODE") {
+        # by code
         while ($done) {
             $result = "";
             $done = sysread($fh, $result, $bufsize);
@@ -75,6 +82,7 @@ sub _file_read_io ($$$$) {
         }
         $result = $data->("");
     } else {
+        # normal
         $result = "";
         while ($done) {
             $done = sysread($fh, $result, $bufsize, length($result));
@@ -85,36 +93,18 @@ sub _file_read_io ($$$$) {
     return(\$result);
 }
 
-my %file_read_options = (
-    binmode => { optional => 1, type => SCALAR, regex => qr/^(binary|utf8)$/ },
-    bufsize => { optional => 1, type => SCALAR, regex => $_IntegerRegexp },
-    data    => { optional => 1, type => SCALARREF | CODEREF },
-);
-
-sub file_read ($@) {
-    my($path, %option, $fh, $result);
-
-    $path = shift(@_);
-    %option = validate(@_, \%file_read_options) if @_;
-    $option{bufsize} ||= $DefaultBufSize;
-    open($fh, "<", $path) or dief("cannot open(%s): %s", $path, $!);
-    _binmode($path, $fh, $option{binmode}) if $option{binmode};
-    $result = _file_read_io($path, $fh, $option{data}, $option{bufsize});
-    close($fh) or dief("cannot close(%s): %s", $path, $!);
-    return(${ $result });
-}
-
 #
-# write to a file
+# syswrite() helper
 #
 
-sub _file_write_io ($$$$) {
+sub _write ($$$$) {
     my($path, $fh, $data, $bufsize) = @_;
     my($ref, $offset, $length, $done, $chunk);
 
     $offset = 0;
     $ref = ref($data);
     if ($ref eq "SCALAR") {
+        # by reference
         $length = length(${ $data });
         while ($length) {
             $done = syswrite($fh, ${ $data }, $bufsize, $offset);
@@ -124,6 +114,7 @@ sub _file_write_io ($$$$) {
             $offset += $done;
         }
     } elsif ($ref eq "CODE") {
+        # by code
         while (1) {
             $chunk = $data->();
             $length = length($chunk);
@@ -138,6 +129,7 @@ sub _file_write_io ($$$$) {
             }
         }
     } else {
+        # normal
         $length = length($data);
         while ($length) {
             $done = syswrite($fh, $data, $bufsize, $offset);
@@ -149,9 +141,42 @@ sub _file_write_io ($$$$) {
     }
 }
 
-my %file_write_options = (
-    binmode => { optional => 1, type => SCALAR, regex => qr/^(binary|utf8)$/ },
+#
+# common read/write options
+#
+
+my %file_rw_options = (
+    binary  => { optional => 1, type => BOOLEAN },
+    binmode => { optional => 1, type => SCALAR },
     bufsize => { optional => 1, type => SCALAR, regex => $_IntegerRegexp },
+    handle  => { optional => 1, type => HANDLE },
+);
+
+#
+# read from a file
+#
+
+my %file_read_options = (%file_rw_options,
+    data    => { optional => 1, type => SCALARREF | CODEREF },
+);
+
+sub file_read ($@) {
+    my($path, %option, $fh, $result);
+
+    $path = shift(@_);
+    %option = validate(@_, \%file_read_options) if @_;
+    $option{bufsize} ||= $DefaultBufSize;
+    $fh = _open($path, "<", \%option);
+    $result = _read($path, $fh, $option{data}, $option{bufsize});
+    close($fh) or dief("cannot close(%s): %s", $path, $!);
+    return(${ $result });
+}
+
+#
+# write to a file
+#
+
+my %file_write_options = (%file_rw_options,
     data    => { optional => 0, type => SCALAR | SCALARREF | CODEREF },
 );
 
@@ -161,9 +186,8 @@ sub file_write ($@) {
     $path = shift(@_);
     %option = validate(@_, \%file_write_options);
     $option{bufsize} ||= $DefaultBufSize;
-    open($fh, ">", $path) or dief("cannot open(%s): %s", $path, $!);
-    _binmode($path, $fh, $option{binmode}) if $option{binmode};
-    _file_write_io($path, $fh, $option{data}, $option{bufsize});
+    $fh = _open($path, ">", \%option);
+    _write($path, $fh, $option{data}, $option{bufsize});
     close($fh) or dief("cannot close(%s): %s", $path, $!);
 }
 
@@ -227,12 +251,16 @@ read the file at the given path and return its contents; supported options:
 
 =over
 
-=item * C<binmode>: binary mode to use (see below)
+=item * C<binary>: treat the file as binary
+
+=item * C<binmode>: binary mode to use
 
 =item * C<bufsize>: buffer size to use for I/O operations
 
 =item * C<data>: return the file contents via this scalar reference or code
-reference (see below)
+reference
+
+=item * C<handle>: file handle to use
 
 =back
 
@@ -242,12 +270,16 @@ write the given contents to the file at the given path; supported options:
 
 =over
 
-=item * C<binmode>: binary mode to use (see below)
+=item * C<binary>: treat the file as binary
+
+=item * C<binmode>: binary mode to use
 
 =item * C<bufsize>: buffer size to use for I/O operations
 
 =item * C<data>: provide the file contents via this scalar, scalar reference
-or code reference (see below)
+or code reference
+
+=item * C<handle>: file handle to use
 
 =back
 
@@ -255,28 +287,28 @@ or code reference (see below)
 
 =head1 OPTIONS
 
-All the functions support a C<binmode> option specifying how the file should
-be accessed:
+Both functions support a C<handle> option that can contain a file handle to
+use. When given, this handle will be used (and closed at the end of the I/O
+operations) as is, without calling binmode() on it (see below).
 
-=over
+These functions also support a C<binary> option and a C<binmode> option
+specifying how the file handle should be treated with respect to binmode().
 
-=item * C<binary>: binmode(FH) will be used to treat the file as binary
+If C<binmode> is set, binmode() will be used with the given layer.
 
-=item * C<utf8>: binmode(FH, ":encoding(utf8)") will be used to select UTF-8
-encoding
+If C<binmode> is not set but C<binary> is true, binmode() will be used without
+any layer.
 
-=item * otherwise: binmode() will not be used (this is the default)
+If neither C<binmode> nor C<binary> are set, binmode() will not be used. This
+is the default.
 
-=back
-
-file_read() can be given a code reference via the C<data> option.  Each time
-data is read via sysread(), the subroutine will be called with the read
-data. At the end of the file, the subroutine will be called with an empty
-string.
+file_read() can be given a code reference via the C<data> option. Each time
+data is read via sysread(), the subroutine will be called with the read data.
+At the end of the file, the subroutine will be called with an empty string.
 
 file_write() can be given a code reference via the C<data> option. It should
 return data in a way similar to sysread(), returning an empty string to
-indicate the end of the data to write to the file.
+indicate the end of the data to be written to the file.
 
 =head1 GLOBAL VARIABLES
 
