@@ -13,8 +13,8 @@
 package No::Worries::Warn;
 use strict;
 use warnings;
-our $VERSION  = "1.3";
-our $REVISION = sprintf("%d.%02d", q$Revision: 1.21 $ =~ /(\d+)\.(\d+)/);
+our $VERSION  = "1.4";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.22 $ =~ /(\d+)\.(\d+)/);
 
 #
 # used modules
@@ -137,7 +137,7 @@ spaces.
 
 It also provides a handler for warn() that prepends a prefix
 ($No::Worries::Warn::Prefix) to all warnings. It also uses the
-NO_WORRIES environment variable to find out if L<Carp>'s carp() or
+C<NO_WORRIES> environment variable to find out if L<Carp>'s carp() or
 cluck() should be used instead of warn(). Finally, the wrapper can be
 told to also log warnings to syslog (see $No::Worries::Warn::Syslog).
 
@@ -191,8 +191,8 @@ L<No::Worries::Syslog>'s syslog_warning() (default: false)
 
 =head1 ENVIRONMENT VARIABLES
 
-This module uses the C<NO_WORRIES> environment variable to find out
-how warnings will be reported. Supported values are:
+This module uses the C<NO_WORRIES> environment variable to control how
+warnings should be reported. Supported values are:
 
 =over
 
